# -*- coding: utf-8 -*-
u"""Release data for the pyreadline project.

$Id$"""

#*****************************************************************************
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************

# Name of the package for release purposes.  This is the name which labels
# the tarballs and RPMs made by distutils, so it's best to lowercase it.
name = u'pyreadline'

# For versions with substrings (like 0.6.16.svn), use an extra . to separate
# the new substring.  We have to avoid using either dashes or underscores,
# because bdist_rpm does not accept dashes (an RPM) convention, and
# bdist_deb does not accept underscores (a Debian convention).

branch = u''

version = u'1.7.1'

revision = u'$Revision$'

description = u"A python implmementation of GNU readline."

long_description = \
u"""
The pyreadline package is a python implementation of GNU readline functionality
it is based on the ctypes based UNC readline package by Gary Bishop. 
It is not complete. It has been tested for use with windows 2000 and windows xp.

Version 1.7.1 includes fixes for 64-bit windows, thanks to Christoph Gohlke for
helping out.

Version 1.7 will be the last release with compatibility with 2.4 and 2.5. The next
major release will target 2.6, 2.7 and 3.x. The 1.7 series will only receive bugfixes
from now on.

Features:
 *  keyboard text selection and copy/paste
 *  Shift-arrowkeys for text selection
 *  Control-c can be used for copy activate with allow_ctrl_c(True) in config file
 *  Double tapping ctrl-c will raise a KeyboardInterrupt, use ctrl_c_tap_time_interval(x)
    where x is your preferred tap time window, default 0.3 s.
 *  paste pastes first line of content on clipboard. 
 *  ipython_paste, pastes tab-separated data as list of lists or numpy array if all data is numeric
 *  paste_mulitline_code pastes multi line code, removing any empty lines.
 
 
 The latest development version is always available at the IPython subversion
 repository_.

.. _repository:
 """

license = u'BSD'

authors = {u'Jorgen' : (u'Jorgen Stenarson',u'jorgen.stenarson@bostream.nu'),
           u'Gary':    (u'Gary Bishop', ''),         
           u'Jack':    (u'Jack Trainor', ''),         
           }

url = u'http://ipython.scipy.org/moin/PyReadline/Intro'

download_url = u'https://launchpad.net/pyreadline/+download'

platforms = [u'Windows XP/2000/NT',
             u'Windows 95/98/ME']

keywords = [u'readline',
            u'pyreadline']

classifiers = [u'Development Status :: 5 - Production/Stable',
               u'Environment :: Console',
               u'Operating System :: Microsoft :: Windows',
               u'License :: OSI Approved :: BSD License',
               u'Programming Language :: Python :: 2.4',
               u'Programming Language :: Python :: 2.5',
               u'Programming Language :: Python :: 2.6',
               u'Programming Language :: Python :: 2.7',
               ]
               
               



"""
A simple test to make sure seash's parser is catching wrong commands correctly.
"""

#pragma error Command not understood
import seash_dictionary

# Parser should recognize these commands as valid
seash_dictionary.parse_command("loadkeys notakey")
seash_dictionary.parse_command("show ip to example_file.txt")
seash_dictionary.parse_command("run example.1.1.r2py example arguments")

# Parser should throw an error for following command
seash_dictionary.parse_command("incorrect command")



"""
Executes a few help commands and make sures seash prints out the correct output.
Make sure to update the txt file of the expected output if help's contents have
modified
"""
import seash
import sys


orig_stdout = sys.stdout
sys.stdout = open("test_results.txt", "w")
command_list = [
  'help', 
  'help extended', 
  'help show', 
  'help show info', 
  'help show ip to file'
  ]

seash.command_loop(command_list)

sys.stdout.close()


# Compares the test results with the expected text that should have been printed
# out when executing the series of help commands
sys.stdout = orig_stdout

test_results = open("test_results.txt", "r")
wanted_results = open("help_test_results.txt", "r")

original = wanted_results.readlines()
actual = test_results.readlines()

for i in range(len(original)):
  if not original[i] == actual[i]:
    print "Line " + str(i) + " of test results are not consistent with expected results: help_test_results.txt"

test_results.close()
wanted_results.close()




""" 
Author: Justin Cappos
Edited: Alan Loh

Module: A shell for Seattle called seash (pronounced see-SHH).   It's not meant
        to be the perfect shell, but it should be good enough for v0.1

Start date: September 18th, 2008

This is an example experiment manager for Seattle.   It allows a user to 
locate vessels they control and manage those vessels.

The design goals of this version are to be secure, simple, and reliable (in 
that order).   

Note: I've written this assuming that repy <-> python integration is perfect
and transparent (minus the bit of mess that fixes this).   As a result this
code may change significantly in the future.


Editor's Note: A large portion of seash's code has been separated into
different files. The only code remaining is the command loop that handles the 
user's input and the exception handling.

Command functions have been moved to command_callbacks.py. 

Helper functions and functions that operate on a target have been moved to 
seash_helper.py.

The newly implemented command dictionary, input parsing, and command dispatching 
have been moved to seash_dictionary.py.

Certain global variables are now kept track in seash_global_variables.py.

Besides the restructuring, the main change that has been made on seash is 
command input are now parsed according to the command dictionary. In addition,
with a structured database of sorts for commands, it should now be easier to 
implement new functions into seash as long as the correct format is followed in
implementing new commands.
"""


# Let's make sure the version of python is supported
import checkpythonversion
checkpythonversion.ensure_python_version_is_supported()

# Warnings are turned on to display the tab completion warning
import warnings
warnings.simplefilter("default")

# simple client.   A better test client (but nothing like what a real client
# would be)

# Restores the original python built-in type() to certain built-in libraries
# as repyportability destroys it.

# Needed for parsing user commands and executing command functions
import seash_dictionary

### Integration fix here...
from repyportability import *

# We need to expose the readline object file to OSX because the default object
# file for Python 2.7 on OSX is not compatible with our tab completion module.
import os
import sys

# Only rename if we're running on OSX
rename_readline_so_file = sys.platform == 'darwin'
HIDDEN_READLINE_SO_FN = 'readline.so.mac'
EXPOSED_READLINE_SO_FN = 'readline.so'

# Make sure we don't overwrite an existing readline.so if it exists.
# We need to do this because os.rename() doesn't raise any errors
# if the destination file already exists.
if (rename_readline_so_file and 
    EXPOSED_READLINE_SO_FN not in os.listdir('.')):
  try:
    os.rename(HIDDEN_READLINE_SO_FN, EXPOSED_READLINE_SO_FN)
  except OSError:
    # There was a problem reading readline.so.mac
    rename_readline_so_file = False

tabcompletion = True
try:
  try:
    # Required for windows tab-completion. 
    # This is the readline module provided by pyreadline 1.7.1.
    # http://pypi.python.org/pypi/pyreadline
    import readline_windows as readline
  except ImportError:
    # This error occurs when trying to import win_readline on mac/linux.
    # We use the default readline module for mac/linux.
    import readline
  import tab_completer
except ImportError:
  tabcompletion = False
  
# Don't hide mac readline.so if we didn't expose it
if rename_readline_so_file:
  os.rename(EXPOSED_READLINE_SO_FN, HIDDEN_READLINE_SO_FN)

# Used for re-enabling modules on the last run
import seash_modules

# To be able to catch certain exceptions thrown throughout the program
import seash_exceptions

import seash_helper

from repyportability import *
add_dy_support(locals())

dy_import_module_symbols("time.r2py")

import traceback

import os.path    # fix path names when doing upload, loadkeys, etc.




def command_loop(test_command_list):
  
  # If a test command list is passed, filter the tab completion warning
  if test_command_list:
    warnings.filterwarnings("ignore", "Auto tab completion is off, because it is not available on your operating system.",
                            ImportWarning)


  # Things that may be set herein and used in later commands.
  # Contains the local variables of the original command loop.
  # Keeps track of the user's state in seash. Referenced 
  # during command executions by the command_parser.
  environment_dict = {
    'host': None, 
    'port': None, 
    'expnum': None,
    'filename': None,
    'cmdargs': None,
    'defaulttarget': None,
    'defaultkeyname': None,
    'currenttarget': None,
    'currentkeyname': None,
    'autosave': False,
    'handleinfo': {},
    'showparse': True,
    }

  

  # Set up the tab completion environment (Added by Danny Y. Huang)
  if tabcompletion:
    # Initializes seash's tab completer
    completer = tab_completer.Completer()
    readline.parse_and_bind("tab: complete")
    # Determines when a new tab complete instance should be initialized,
    # which, in this case, is never, so the tab completer will always take
    # the entire user's string into account
    readline.set_completer_delims("")
    # Sets the completer function that readline will utilize
    readline.set_completer(completer.complete)
  else:
    warnings.warn("Auto tab completion is off, because it is not available on your operating system.",ImportWarning)


  # If passed a list of commands, do not prompt for user input
  if test_command_list:
    seash_helper.update_time()
    # Iterates through test_command_list in sequential order
    for command_strings in test_command_list:
      # Saving state after each command? (Added by Danny Y. Huang)
      if environment_dict['autosave'] and environment_dict['defaultkeyname']:
        try:
          # State is saved in file "autosave_username", so that user knows which
          # RSA private key to use to reload the state.
          autosavefn = "autosave_" + str(environment_dict['defaultkeyname'])
          seash_helper.savestate(autosavefn, environment_dict['handleinfo'], environment_dict['host'], 
                                 environment_dict['port'], environment_dict['expnum'], 
                                 environment_dict['filename'], environment_dict['cmdargs'], 
                                 environment_dict['defaulttarget'], environment_dict['defaultkeyname'], 
                                 environment_dict['autosave'], environment_dict['defaultkeyname'])
        except Exception, error:
          raise seash_exceptions.UserError("There is an error in autosave: '" + str(error) + "'. You can turn off autosave using the command 'set autosave off'.")

      # Returns the dictionary of dictionaries that correspond to the
      # command string
      cmd_input = seash_dictionary.parse_command(command_strings, display_parsed_result=environment_dict['showparse'])
      
      # by default, use the target specified in the prompt
      environment_dict['currenttarget'] = environment_dict['defaulttarget']
      
      # by default, use the identity specified in the prompt
      environment_dict['currentkeyname'] = environment_dict['defaultkeyname']

      # calls the command_dispatch method of seash_dictionary to execute the callback
      # method associated with the command the user inputed
      seash_dictionary.command_dispatch(cmd_input, environment_dict)



  # Otherwise launch into command loop, exit via return
  else:
    while True:
      try:
        if tabcompletion:
          # Updates the list of variable values in the tab complete class
          completer.set_target_list()
          completer.set_keyname_list()
          
        # Saving state after each command? (Added by Danny Y. Huang)
        if environment_dict['autosave'] and environment_dict['defaultkeyname']:
          try:
            # State is saved in file "autosave_username", so that user knows which
            # RSA private key to use to reload the state.
            autosavefn = "autosave_" + str(environment_dict['defaultkeyname'])
            seash_helper.savestate(autosavefn, environment_dict['handleinfo'], environment_dict['host'], 
                                   environment_dict['port'], environment_dict['expnum'], 
                                   environment_dict['filename'], environment_dict['cmdargs'], 
                                   environment_dict['defaulttarget'], environment_dict['defaultkeyname'], 
                                   environment_dict['autosave'], environment_dict['defaultkeyname'])
          except Exception, error:
            raise seash_exceptions.UserError("There is an error in autosave: '" + str(error) + "'. You can turn off autosave using the command 'set autosave off'.")


        prompt = ''
        if environment_dict['defaultkeyname']:
          prompt = seash_helper.fit_string(environment_dict['defaultkeyname'],20)+"@"

        # display the thing they are acting on in their prompt (if applicable)
        if environment_dict['defaulttarget']:
          prompt = prompt + seash_helper.fit_string(environment_dict['defaulttarget'],20)

        prompt = prompt + " !> "
        # the prompt should look like: justin@good !> 
        
        # get the user input
        userinput = raw_input(prompt)
        
        # allows commenting
        comment_index = userinput.find('#')

        # throw away anything after (and including) the '#'
        if comment_index != -1:
          userinput = userinput[0:comment_index]

        # drop any leading or trailing whitespace
        userinput = userinput.strip()

        # if it's an empty line, continue...
        if len(userinput)==0:
          continue
      
        # Returns the dictionary of dictionaries that correspond to the
        # command the user inputted
        cmd_input = seash_dictionary.parse_command(userinput, display_parsed_result=environment_dict['showparse'])
      
      
        # by default, use the target specified in the prompt
        environment_dict['currenttarget'] = environment_dict['defaulttarget']
        
        # by default, use the identity specified in the prompt
        environment_dict['currentkeyname'] = environment_dict['defaultkeyname']

        # calls the command_dispatch method of seash_dictionary to execute the callback
        # method associated with the command the user inputed
        seash_dictionary.command_dispatch(cmd_input, environment_dict)


 

# handle errors
      except KeyboardInterrupt:
        # print or else their prompt will be indented
        print
        # Make sure the user understands why we exited
        print 'Exiting due to user interrupt'
        return
      except EOFError:
        # print or else their prompt will be indented
        print
        # Make sure the user understands why we exited
        print 'Exiting due to EOF (end-of-file) keystroke'
        return

      except seash_exceptions.ParseError, error_detail:
        print 'Invalid command input:', error_detail
      except seash_exceptions.DispatchError, error_detail:
        print error_detail
      except seash_exceptions.UserError, error_detail: 
        print error_detail
      except SystemExit:
        # exits command loop
        return
      except:
        traceback.print_exc()
      
  
  
if __name__=='__main__':
  seash_helper.update_time()
  seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)
  
  # For general usage, empty list is passed to prompt for user input
  command_loop([])



""" 
Author: Justin Cappos

Module: Node Manager initializer.   It initializes the state needed to run the
        node manager on the local node.   This would most likely be run by the
        installer.

Start date: September 10rd, 2008

This initializes the node manager for Seattle.   It sets up the starting 
resources, creates a configuration file, etc.

The design goals of this version are to be secure, simple, and reliable (in 
that order).   

"""

from repyportability import *
add_dy_support(locals())

# need to generate a public key
dy_import_module_symbols('rsa.r2py')

# need randomfloat...
import random
randomfloat = random.random


import os

import persist

import shutil

import glob

# embedded here.   Is this really the right thing to do?
justinpubkey = {'e':22599311712094481841033180665237806588790054310631222126405381271924089573908627143292516781530652411806621379822579071415593657088637116149593337977245852950266439908269276789889378874571884748852746045643368058107460021117918657542413076791486130091963112612854591789518690856746757312472362332259277422867, 'n':12178066700672820207562107598028055819349361776558374610887354870455226150556699526375464863913750313427968362621410763996856543211502978012978982095721782038963923296750730921093699612004441897097001474531375768746287550135361393961995082362503104883364653410631228896653666456463100850609343988203007196015297634940347643303507210312220744678194150286966282701307645064974676316167089003178325518359863344277814551559197474590483044733574329925947570794508677779986459413166439000241765225023677767754555282196241915500996842713511830954353475439209109249856644278745081047029879999022462230957427158692886317487753201883260626152112524674984510719269715422340038620826684431748131325669940064404757120601727362881317222699393408097596981355810257955915922792648825991943804005848347665699744316223963851263851853483335699321871483966176480839293125413057603561724598227617736944260269994111610286827287926594015501020767105358832476708899657514473423153377514660641699383445065369199724043380072146246537039577390659243640710339329506620575034175016766639538091937167987100329247642670588246573895990251211721839517713790413170646177246216366029853604031421932123167115444834908424556992662935981166395451031277981021820123445253}

# Vessels need to have a public key in order to be accessed
print "Generating user keys..."
keylen = 2 ** 10
publickeys = []

# Our unit tests need access to 4 guest users, from guest0...guest3
num_guests = 4

for i in range(num_guests):
  publickey, privatekey = rsa_gen_pubpriv_keys(keylen)
  publickeys.append(publickey)
  # The unit tests need access to these keys
  publickey_file = 'guest' + str(i) + '.publickey'
  privatekey_file = 'guest' + str(i) + '.privatekey'

  # We need to make sure the file that we're writing to doesn't yet
  # exist.
  for filename in (publickey_file, privatekey_file):
    # We shouldn't rely on querying if the files exist before
    # deleting, as someone can get to the file after we query
    # and before we delete.
    try:
      removefile(filename)
    except FileNotFoundError:
      pass

  rsa_publickey_to_file(publickey, publickey_file)
  rsa_privatekey_to_file(privatekey, privatekey_file)

( guest0pubkey,
  guest1pubkey,
  guest2pubkey,
  guest3pubkey) = publickeys


# This is the public key of the person who will control most of the resources.
controllerpubkey = {'e': 1515278400394037168869631887206225761783197636247636149274740854708478416229147500580877416652289990968676310353790883501744269103521055894342395180721167L, 'n': 8811850224687278929671477591179591903829730117649785862652866020803862826558480006479605958786097112503418194852731900367494958963787480076175614578652735061071079458992502737148356289391380249696938882025028801032667062564713111819847043202173425187133883586347323838509679062142786013585264788548556099117804213139295498187634341184917970175566549405203725955179602584979965820196023950630399933075080549044334508921319264315718790337460536601263126663173385674250739895046814277313031265034275415434440823182691254039184953842629364697394327806074576199279943114384828602178957150547925812518281418481896604655037L}



offcutresourcedata ="""# BUG: How do we come up with these values dynamically?
resource cpu .002
resource memory 1000000   # 1 MiB
resource diskused 100000 # .1 MiB
resource events 2
resource filewrite 1000
resource fileread 1000 
resource filesopened 1 
resource insockets 0
resource outsockets 0
resource netsend 0
resource netrecv 0
resource loopsend 0  # would change with prompt functionality (?)
resource looprecv 0
resource lograte 100 # the monitor might log something
resource random 0    # Shouldn't generate random numbers on our own
"""

bigresourcedata = """resource cpu .08
resource memory 100000000   # 100 MiB
resource diskused 80000000 # 80 MiB
resource events 50
resource filewrite 100000
resource fileread 100000
resource filesopened 10
resource insockets 10
resource outsockets 10
resource netsend 100000
resource netrecv 100000
resource loopsend 1000000
resource looprecv 1000000
resource lograte 30000
resource random 100
resource messport 11111
resource messport 12222
resource messport 13333
resource messport 14444
resource messport 15555
resource messport 16666
resource messport 17777
resource messport 18888
resource messport 19999
resource connport 11111
resource connport 12222
resource connport 13333
resource connport 14444
resource connport 15555
resource connport 16666
resource connport 17777
resource connport 18888
resource connport 19999

call gethostbyname_ex allow
call sendmess allow
call recvmess allow
call openconn allow
call waitforconn allow
call stopcomm allow                     # it doesn't make sense to restrict
call socket.close allow                 # let's not restrict
call socket.send allow                  # let's not restrict
call socket.recv allow                  # let's not restrict

# open and file.__init__ both have built in restrictions...
call open allow                         # can read / write
call file.__init__ allow                # can read / write
call file.close allow                   # shouldn't restrict
call file.flush allow                   # they are free to use
call file.next allow                    # free to use as well...
call file.read allow                    # allow read
call file.readline allow                # shouldn't restrict
call file.readlines allow               # shouldn't restrict
call file.seek allow                    # seek doesn't restrict
call file.write allow                   # shouldn't restrict (open restricts)
call file.writelines allow              # shouldn't restrict (open restricts)
call sleep allow                        # harmless
call settimer allow                     # we can't really do anything smart
call canceltimer allow                  # should be okay
call exitall allow                      # should be harmless 

call log.write allow
call log.writelines allow
call getmyip allow                      # They can get the external IP address
call listdir allow                      # They can list the files they created
call removefile allow                   # They can remove the files they create
call randomfloat allow                  # can get random numbers
call getruntime allow                   # can get the elapsed time
call getlock allow                      # can get a mutex
"""

smallresourcedata = """resource cpu .02
resource memory 30000000   # 30 MiB
resource diskused 20000000 # 20 MiB
resource events 15
resource filewrite 100000
resource fileread 100000
resource filesopened 5
resource insockets 5
resource outsockets 5
resource netsend 10000
resource netrecv 10000
resource loopsend 1000000
resource looprecv 1000000
resource lograte 30000
resource random 100
resource messport %s
resource messport %s
resource messport %s
resource messport %s
resource connport %s
resource connport %s
resource connport %s
resource connport %s

call gethostbyname_ex allow
call sendmess allow
call recvmess allow
call openconn allow
call waitforconn allow
call stopcomm allow                     # it doesn't make sense to restrict
call socket.close allow                 # let's not restrict
call socket.send allow                  # let's not restrict
call socket.recv allow                  # let's not restrict

# open and file.__init__ both have built in restrictions...
call open allow                         # can read / write
call file.__init__ allow                # can read / write
call file.close allow                   # shouldn't restrict
call file.flush allow                   # they are free to use
call file.next allow                    # free to use as well...
call file.read allow                    # allow read
call file.readline allow                # shouldn't restrict
call file.readlines allow               # shouldn't restrict
call file.seek allow                    # seek doesn't restrict
call file.write allow                   # shouldn't restrict (open restricts)
call file.writelines allow              # shouldn't restrict (open restricts)
call sleep allow                        # harmless
call settimer allow                     # we can't really do anything smart
call canceltimer allow                  # should be okay
call exitall allow                      # should be harmless 

call log.write allow
call log.writelines allow
call getmyip allow                      # They can get the external IP address
call listdir allow                      # They can list the files they created
call removefile allow                   # They can remove the files they create
call randomfloat allow                  # can get random numbers
call getruntime allow                   # can get the elapsed time
call getlock allow                      # can get a mutex
"""





def make_vessel(vesselname, pubkey, resourcetemplate, resourceargs):
  retdict = {'userkeys':[], 'ownerkey':pubkey, 'oldmetadata':None, 'stopfilename':vesselname+'.stop', 'logfilename':vesselname+'.log', 'statusfilename':vesselname+'.status', 'resourcefilename':'resource.'+vesselname, 'advertise':True, 'ownerinformation':'', 'status':'Fresh'}

  try:
    WindowsError

  except NameError: # not on windows...
    # make the vessel dirs...
    try:
      os.mkdir(vesselname)
    except OSError,e:
      if e[0] == 17:
        # directory exists
        pass
      else:
        raise

  else: # on Windows...

    # make the vessel dirs...
    try:
      os.mkdir(vesselname)
    except (OSError,WindowsError),e:
      if e[0] == 17 or e[0] == 183:
        # directory exists
        pass
      else:
        raise


  #### write the vessel's resource file...
  outfo = open(retdict['resourcefilename'],"w")
  # write the args into the resource data template
  outfo.write(resourcetemplate % resourceargs)
  outfo.close()
  
  return retdict



# lots of little things need to be initialized...   
def initialize_state():

  # first, let's clean up any existing directory data...
  for vesseldirectoryname in glob.glob('v[0-9]*'):
    if os.path.isdir(vesseldirectoryname):
      print 'Removing:',vesseldirectoryname
      shutil.rmtree(vesseldirectoryname)

  # initialize my configuration file.   This involves a few variables:
  #    pollfrequency --  the amount of time to sleep after a check when "busy
  #                      waiting".   This trades CPU load for responsiveness.
  #    ports         --  the ports the node manager could listen on.
  #    publickey     --  the public key used to identify the node...
  #    privatekey    --  the corresponding private key for the node...
  configuration = {}

  configuration['pollfrequency'] = 1.0

  # NOTE: I chose these randomly (they will be uniform across all NMs)...   
  # Was this wise?

  # Modified By Monzur on Dec 2, 2010. Changed the first port to the tag
  # nodemanager_port tag. Usually this will be replaced by the port number
  # 1224 by preparetest.py. If --random-ports option is used when running
  # preparetest.py, then the first port will be a random port. This will 
  # allow us to run multiple nodemanagers for testing purposes.
  configuration['ports'] = [<nodemanager_port>, 2888, 9625, 10348, 39303, 48126, 52862, 57344, 64310]

  print "Generating owner key..."
  keys = rsa_gen_pubpriv_keys(100)
  configuration['publickey'] = keys[0]
  configuration['privatekey'] = keys[1]
  configuration['service_vessel'] = 'v2'

  print "Writing config file..."
  # write the config file...
  persist.commit_object(configuration,"nodeman.cfg")

  # write the offcut file...
  outfo = open("resources.offcut","w")
  outfo.write(offcutresourcedata)
  outfo.close()

#  vessel1 = make_vessel('v1',controllerpubkey,bigresourcedata, []) 
  vessel1 = make_vessel('v1',controllerpubkey,smallresourcedata, ('12345','12346', '12347','12348','12345','12346','12347','12348')) 
  vessel2 = make_vessel('v2',justinpubkey,smallresourcedata, ('20000','20001', '20002','20003','20000','20001','20002','20003')) 
  vessel3 = make_vessel('v3',guest0pubkey,smallresourcedata, ('30000','30001', '30002','30003','30000','30001','30002','30003')) 
  vessel4 = make_vessel('v4',guest0pubkey,smallresourcedata, ('21000','21001', '21002','21003','21000','21001','21002','21003')) 
  vessel5 = make_vessel('v5',guest1pubkey,smallresourcedata, ('22000','22001', '22002','22003','22000','22001','22002','22003')) 
  vessel6 = make_vessel('v6',guest1pubkey,smallresourcedata, ('23000','23001', '23002','23003','23000','23001','23002','23003')) 
  vessel7 = make_vessel('v7',guest2pubkey,smallresourcedata, ('24000','24001', '24002','24003','24000','24001','24002','24003')) 
  vessel8 = make_vessel('v8',guest2pubkey,smallresourcedata, ('25000','25001', '25002','25003','25000','25001','25002','25003')) 
  vessel9 = make_vessel('v9',guest3pubkey,smallresourcedata, ('26000','26001', '26002','26003','26000','26001','26002','26003')) 
  vessel10 = make_vessel('v10',guest3pubkey,smallresourcedata, ('27000','27001', '27002','27003','27000','27001','27002','27003')) 
  

  vesseldict = {'v1':vessel1, 'v2':vessel2, 'v3':vessel3, 'v4':vessel4, 'v5':vessel5, 'v6':vessel6, 'v7':vessel7, 'v8':vessel8, 'v9':vessel9, 'v10':vessel10}

  print "Writing vessel dictionary..."
  # write out the vessel dictionary...
  persist.commit_object(vesseldict,"vesseldict")










if __name__ == '__main__':
  initialize_state() 



"""
Test the merging of commanddicts.

"""
# Needed so that the built-in type function is restored to the python
# abc library
import seash_dictionary

import seash_modules
from copy import deepcopy


def nullfunction():
  """ We need to point the callback to an existing function. """
  pass



# Base dictionary that we will make a copy of and run the tests on. 
original_dict = {
  'levelone':{'callback': nullfunction, 'help_text': 'Help 1', 'summary': 'Summary 1', 'children':{
      'leveltwo':{'callback': nullfunction, 'help_text': 'Help 2', 'summary': 'Summary 2', 'children':{}},
  }},
}



def rootlevel_test():
  level_one_dict = {
    'leveloneprime':{'callback': nullfunction, 'help_text': 'Help 1`', 'summary': 'Summary 1`', 'children':{}},
  }
  
  level_one_expected = {
    'levelone':{'callback': nullfunction, 'help_text': 'Help 1', 'summary': 'Summary 1', 'children':{
        'leveltwo':{'callback': nullfunction, 'help_text': 'Help 2', 'summary': 'Summary 2', 'children':{}},
    }},
    'leveloneprime':{'callback': nullfunction, 'help_text': 'Help 1`', 'summary': 'Summary 1`', 'children':{}},
  }
  
  testdict = deepcopy(original_dict)
  seash_modules.merge_commanddict(testdict, level_one_dict)
  if not seash_modules.are_cmddicts_same(testdict, level_one_expected):
    raise Exception("Failed to merge dictionaries on root level")
  
  # Repeat the same test the other way around
  testdict = deepcopy(level_one_dict)
  seash_modules.merge_commanddict(testdict, original_dict) 
  if not seash_modules.are_cmddicts_same(testdict, level_one_expected):
    raise Exception("Failed to merge dictionaries on root level")



def secondlevel_test():
  # Make sure merging on 2nd-level works
  level_two_dict = {
    'levelone':{'children':{
        'leveltwoprime':{'callback': nullfunction, 'help_text': 'Help 2`', 'summary': 'Summary 2`', 'children':{}},
    }},
  }
  
  level_two_expected = {
    'levelone':{'callback': nullfunction, 'help_text': 'Help 1', 'summary': 'Summary 1', 'children':{
        'leveltwo':{'callback': nullfunction, 'help_text': 'Help 2', 'summary': 'Summary 2', 'children':{}},
        'leveltwoprime':{'callback': nullfunction, 'help_text': 'Help 2`', 'summary': 'Summary 2`', 'children':{}},
    }},
  }
  
  testdict = deepcopy(original_dict)
  seash_modules.merge_commanddict(testdict, level_two_dict)
  if not seash_modules.are_cmddicts_same(testdict, level_two_expected):
    import pdb;
    pdb.set_trace()
    raise Exception("Failed to merge dictionaries on 2nd level")
  
  # Repeat the same test the other way around
  testdict = deepcopy(level_two_dict)
  seash_modules.merge_commanddict(testdict, original_dict) 
  if not seash_modules.are_cmddicts_same(testdict, level_two_expected):
    raise Exception("Failed to merge dictionaries on 2nd level")



rootlevel_test()
secondlevel_test()


# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2007  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
import sys

try:
    pyreadline_codepage = sys.stdout.encoding
except AttributeError:        
    # This error occurs when pdb imports readline and doctest has replaced 
    # stdout with stdout collector. We will assume ascii codepage
    pyreadline_codepage = u"ascii"

if pyreadline_codepage is None:  
    pyreadline_codepage = u"ascii"

def ensure_unicode(text):
    u"""helper to ensure that text passed to WriteConsoleW is unicode"""
    if isinstance(text, str):
        try:
            return text.decode(pyreadline_codepage, u"replace")
        except (LookupError, TypeError):
            return text.decode(u"ascii", u"replace")
    return text

def ensure_str(text):
    u"""Convert unicode to str using pyreadline_codepage"""
    if isinstance(text, unicode):
        try:
            return text.encode(pyreadline_codepage, u"replace")
        except (LookupError, TypeError):
            return text.encode(u"ascii", u"replace")
    return text



# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2003-2006 Gary Bishop.
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
import winconstants as c32
from   pyreadline.logger import log
from ctypes import windll
import ctypes
# table for translating virtual keys to X windows key symbols

from common import validkey, KeyPress, make_KeyPress_from_keydescr

code2sym_map = {c32.VK_CANCEL:     u'cancel',
                c32.VK_BACK:       u'backspace',
                c32.VK_TAB:        u'tab',
                c32.VK_CLEAR:      u'clear',
                c32.VK_RETURN:     u'return',
                c32.VK_SHIFT:      u'shift_l',
                c32.VK_CONTROL:    u'control_l',
                c32.VK_MENU:       u'alt_l',
                c32.VK_PAUSE:      u'pause',
                c32.VK_CAPITAL:    u'caps_lock',
                c32.VK_ESCAPE:     u'escape',
                c32.VK_SPACE:      u'space',
                c32.VK_PRIOR:      u'prior',
                c32.VK_NEXT:       u'next',
                c32.VK_END:        u'end',
                c32.VK_HOME:       u'home',
                c32.VK_LEFT:       u'left',
                c32.VK_UP:         u'up',
                c32.VK_RIGHT:      u'right',
                c32.VK_DOWN:       u'down',
                c32.VK_SELECT:     u'select',
                c32.VK_PRINT:      u'print',
                c32.VK_EXECUTE:    u'execute',
                c32.VK_SNAPSHOT:   u'snapshot',
                c32.VK_INSERT:     u'insert',
                c32.VK_DELETE:     u'delete',
                c32.VK_HELP:       u'help',
                c32.VK_F1:         u'f1',
                c32.VK_F2:         u'f2',
                c32.VK_F3:         u'f3',
                c32.VK_F4:         u'f4',
                c32.VK_F5:         u'f5',
                c32.VK_F6:         u'f6',
                c32.VK_F7:         u'f7',
                c32.VK_F8:         u'f8',
                c32.VK_F9:         u'f9',
                c32.VK_F10:        u'f10',
                c32.VK_F11:        u'f11',
                c32.VK_F12:        u'f12',
                c32.VK_F13:        u'f13',
                c32.VK_F14:        u'f14',
                c32.VK_F15:        u'f15',
                c32.VK_F16:        u'f16',
                c32.VK_F17:        u'f17',
                c32.VK_F18:        u'f18',
                c32.VK_F19:        u'f19',
                c32.VK_F20:        u'f20',
                c32.VK_F21:        u'f21',
                c32.VK_F22:        u'f22',
                c32.VK_F23:        u'f23',
                c32.VK_F24:        u'f24',
                c32.VK_NUMLOCK:    u'num_lock,',
                c32.VK_SCROLL:     u'scroll_lock',
                c32.VK_APPS:       u'vk_apps',
                c32.VK_PROCESSKEY: u'vk_processkey',
                c32.VK_ATTN:       u'vk_attn',
                c32.VK_CRSEL:      u'vk_crsel',
                c32.VK_EXSEL:      u'vk_exsel',
                c32.VK_EREOF:      u'vk_ereof',
                c32.VK_PLAY:       u'vk_play',
                c32.VK_ZOOM:       u'vk_zoom',
                c32.VK_NONAME:     u'vk_noname',
                c32.VK_PA1:        u'vk_pa1',
                c32.VK_OEM_CLEAR:  u'vk_oem_clear',
                c32.VK_NUMPAD0:    u'numpad0',
                c32.VK_NUMPAD1:    u'numpad1',
                c32.VK_NUMPAD2:    u'numpad2',
                c32.VK_NUMPAD3:    u'numpad3',
                c32.VK_NUMPAD4:    u'numpad4',
                c32.VK_NUMPAD5:    u'numpad5',
                c32.VK_NUMPAD6:    u'numpad6',
                c32.VK_NUMPAD7:    u'numpad7',
                c32.VK_NUMPAD8:    u'numpad8',
                c32.VK_NUMPAD9:    u'numpad9',
                c32.VK_DIVIDE:     u'divide',
                c32.VK_MULTIPLY:   u'multiply',
                c32.VK_ADD:        u'add',
                c32.VK_SUBTRACT:   u'subtract',
                c32.VK_DECIMAL:    u'vk_decimal'
               }

VkKeyScan = windll.user32.VkKeyScanA

def char_to_keyinfo(char, control=False, meta=False, shift=False):
    k=KeyPress()
    vk = VkKeyScan(ord(char))
    if vk & 0xffff == 0xffff:
        print u'VkKeyScan("%s") = %x' % (char, vk)
        raise ValueError, u'bad key'
    if vk & 0x100:
        k.shift = True
    if vk & 0x200:
        k.control = True
    if vk & 0x400:
        k.meta = True
    k.char=chr(vk & 0xff)
    return k

def make_KeyPress(char, state, keycode):
    control = (state & (4+8)) != 0
    meta = (state & (1+2)) != 0
    shift = (state & 0x10) != 0
    if control and not meta:#Matches ctrl- chords should pass keycode as char
        char = chr(keycode)
    elif control and meta:  #Matches alt gr and should just pass on char
        control = False
        meta = False
    try:
        keyname=code2sym_map[keycode]
    except KeyError:
        keyname = u""
    out = KeyPress(char, shift, control, meta, keyname)
    return out

if __name__==u"__main__":
    import startup
    


import sys

success = False
in_ironpython = u"IronPython" in sys.version

if in_ironpython:
    try:
        from ironpython_keysyms import *
        success = True
    except ImportError, x:
        raise
else:
    try:
        from keysyms import *
        success = True
    except ImportError, x:
        pass
    
if not success:
    raise ImportError(u"Could not import keysym for local pythonversion", x)


"""
Make sure that showing information for a nonexistant module fails
"""
import seash
import seash_dictionary
import seash_modules


#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

#pragma error Module is not installed
commands = [
  "modulehelp nonexistantmodule",
]
seash.command_loop(commands)


"""
<Program Name>
  clearinghouse/clearinghouse.py

<Purpose>
  Provides functions to interact with the Clearinghouse.
  
  module_level_help contains instructions on how to use this module from the
  seash command line.
  
"""

import command_callbacks


module_level_help = """
Clearinghouse Module

This module contains commands that allow you to interact with the Seattle 
Clearinghouse.  Specifically, it provides functionality for acquiring and
releasing vessels.

To acquire vessels, use the 'get' command:
  user@ !> get 3

To release vessels, use the 'release' command.
  user@ !> 

Note: Connecting to the Clearinghouse securely via SSL requires M2Crypto
to be installed. In addition, you must have a CA certs file in the
modules/clearinghouse/ directory, named 'cacert.pem'. You may download
this file here: http://curl.haxx.se/ca/cacert.pem. Otherwise, you can
connect insecurely by specifying 'get insecure' or 'release insecure',
though this is not recommended.

"""


get_helptext = """
get # [type]

Connects to the Clearinghouse and acquires vessels.  The identity specified
must have a private key loaded.

guest0@ !> get 3 wan
['129.97.74.14:1224']
Added targets: %1(129.97.74.14:1224:v10), %2(129.97.74.14:1224:v4), %3(129.97.74.14:1224:v6)
Added group 'acquired' with 3 targets
"""

release_helptext = """
release [group]

Releases the vessels belonging to the specified identity.  The vessels must be owned by the
default identity.  If the groupname is omitted, the current group is released.

guest0@ !> browse
['129.97.74.14:1224']
Added targets: %1(129.97.74.14:1224:v10), %2(129.97.74.14:1224:v4), %3(129.97.74.14:1224:v6)
Added group 'browsegood' with 3 targets
guest0@ !> release browsegood
guest0@ !> on browsegood list
guest0@ !>

"""

_get_arguments = {
  'name':'vesselcount', 'callback': command_callbacks.get, 'children':{
    'wan': { 'name':'type', 'callback': command_callbacks.get, 'children':{}},
    'lan': { 'name':'type', 'callback': command_callbacks.get, 'children':{}},
    'nat': { 'name':'type', 'callback': command_callbacks.get, 'children':{}},
  }
}

_release_arguments = {
  'name':'groupname', 'callback': command_callbacks.release_args, 'children': {}
}

command_dict = {
  # get # [type]
  'get':{ 'name':'get', 'callback': None,
             'summary': "Acquires vessels", 'example': "[insecure] # [type]",
             'help_text': get_helptext, 'children': {
      'insecure': {
        'name':'insecure', 'callback': None,
        'summary': "Connect to the clearinghouse insecurely",
        'help_text': get_helptext, 'children': {'[ARGUMENT]': _get_arguments}
      },
      '[ARGUMENT]': _get_arguments,
    }},

  # release group
  'release': { 'name':'release', 'callback': command_callbacks.release,
               'summary': "Releases a group of vessels", 'example': "group",
               'help_text': release_helptext, 'children': {
      'insecure': {
        'name':'insecure', 'callback': None,
        'summary': "Connect to the clearinghouse insecurely",
        'help_text': release_helptext, 'children': {'[ARGUMENT]': _release_arguments}
      },
      '[GROUP]': _release_arguments,
    }}
}

moduledata = {
  'command_dict': command_dict,
  'help_text': module_level_help,
  'url': None,
}


"""
<Program Name>
  modules/__init__.py

<Purpose>
  Provides module manipulation functionality for seash.
  
  module_level_help contains instructions on how to use this module from the
  seash command line.
  
"""


import seash_modules
import seash_dictionary
import seash_helper
import seash_exceptions

# This is where the module callbacks are defined
import command_callbacks


def autocomplete(input_list):
  """
  <Purpose>
    Returns all valid input completions for the specified command line input.
  
  <Arguments>
    input_list: A list of tokens.
  
  <Side Effects>
    None
  
  <Exceptions>
    None
    
  <Returns>
    A list of strings representing valid completions.
  """
  if input_list[0] in ['modulehelp', 'enable', 'disable']:
    commands = []
    for modulename in seash_modules.module_data.keys():
      commands.append(input_list[0] + ' ' + modulename)
    return commands
  return []



module_level_help = """
Modules Module

This module contains commands that pertain to the manipulation of modules.
You can enable/disable modules that are used within seash from here.

To enable a module, use the 'enable' command:
  user@ !> enable variables

To disable a module, use the 'disable' command.
  user@ !> disable variables

You can also view module-level help:
  user@ !> modulehelp variables

And finally, view information on all modules you currently have installed:
  user@ !> show modules

"""


list_all_modules_helptext = """
show modules

Shows information on all installed modules.  Modules will be separated into
two categories: enabled and disabled.  Each entry will be in the following
format:
  [module name] - [URL where the module was installed from, if available]

The URL will be missing if the module was installed manually.


user@ !> show modules
Enabled Modules:
geoip

Installed Modules:
geoip - Install URL not available
selexor - https://seattle.poly.edu/plugins/selexor/
"""

enable_helptext = """
enable [modulename]

Enables use of the specified module.  You can only enable modules if they do
not contain commands that conflict with existing commands.

user@ !> enable modulename
user@ !> enable modulename
Module 'modulename' is already enabled.
user@ !> enable conflictingmodule
Module 'conflictingmodule' cannot be enabled due to these conflicting commands:
show info (default)
get (selexor)
"""

disable_helptext = """
disable [modulename]

Disables the specified module.  You will no longer be able to access the
commands that were found in the disabled module until the module is re-enabled.

Important note:  You cannot disable this module through the disable command.

user@ !> disable modulename
user@ !> disable modulename
Module 'modulename' is not enabled.

"""

modulehelp_helptext = """
help module [modulename]

Displays the module-level help for a particular module.  The module must 
already be installed.

"""

command_dict = {
  # show modules
  'show': {'children': {
      'modules': { 'name': 'modules', 'callback': command_callbacks.list_all_modules,
            'summary': "Shows basic information about all installed modules.",
            'help_text': list_all_modules_helptext, 'children': {}}, }},

  # enable modulename
  'enable':{ 'name':'enable', 'callback': command_callbacks.enable_module,
             'summary': "Enables an installed module", 'example': "[modulename]",
             'help_text': enable_helptext, 'children': {
      '[ARGUMENT]': { 'name':'modulename', 'callback': None, 'children':{}},}},

  # disable modulename
  'disable': { 'name':'disable', 'callback': command_callbacks.disable_module,
               'summary': "Disables an enabled module", 'example': "[modulename]",
               'help_text': disable_helptext, 'children': {
      '[ARGUMENT]': { 'name':'modulename', 'callback': None, 'children':{}},}},

  # modulehelp modulename
  'modulehelp': { 'name': 'modulehelp', 'callback': command_callbacks.print_module_help,
                  'summary': "Shows the module-level helptext for a particular module",
                  'example': '[modulename]', 'help_text': modulehelp_helptext, 
                  'children': {
      '[ARGUMENT]':{ 'name':'modulename', 'callback': None, 'children':{}},}},
}

moduledata = {
  'command_dict': command_dict,
  'help_text': module_level_help,
  'url': None,
  'tab_completer': autocomplete
}


"""
Test that we can acquire and release vessels in secure mode.

"""
import seash
import sys

# We need the modules module
import seash_modules
import seash_dictionary
#pragma out Added group 'acquired' with 5 targets
#pragma out acquired (empty)

seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

commands = [
  'enable clearinghouse',
  'loadkeys seash_gettest',
  'as seash_gettest',
  'get 5',
  'on acquired',
  'release acquired',
  'show targets',
]

# Allow the geoip module to be disabled when done,
# even if exceptions occur
try:
  seash.command_loop(commands)
except Exception, e:
  print>>sys.stderr, str(e)

# Make sure the geoip module is disabled after this run.
# This is to ensure that other modules' tests are not affected by this module.
seash.command_loop(['disable clearinghouse'])



"""
Test the merging of commanddicts.

"""
# Needed so that the built-in type function is restored to the python
# abc library
import seash_dictionary

import seash_modules
from copy import deepcopy


def nullfunction():
  """ We need to point the callback to an existing function. """
  pass


# Base dictionary that we will make a copy of and run the tests on.
COMBINED_DICT = {
  'show':{'callback': nullfunction, 'help_text': 'Help show', 'summary': 'Summary show', 'children':{
      'x':{'callback': nullfunction, 'help_text': 'Help show x', 'summary': 'Summary show x', 'children':{
      }},
      'y':{'callback': nullfunction, 'help_text': 'Help show y', 'summary': 'Summary show y', 'children':{
      }},
  }},
  'get':{'callback': nullfunction, 'help_text': 'Help get', 'summary': 'Summary get', 'children':{
      'x':{'callback': nullfunction, 'help_text': 'Help get x', 'summary': 'Summary get x', 'children':{
      }},
      'y':{'callback': nullfunction, 'help_text': 'Help get y', 'summary': 'Summary get y', 'children':{
      }},
  }},
}

# This dictionary contains all the get-commands.
GET_DICT = {
  'get':{'callback': nullfunction, 'help_text': 'Help get', 'summary': 'Summary get', 'children':{
      'x':{'callback': nullfunction, 'help_text': 'Help get x', 'summary': 'Summary get x', 'children':{
      }},
      'y':{'callback': nullfunction, 'help_text': 'Help get y', 'summary': 'Summary get y', 'children':{
      }},
  }},
}

# This dictionary contains all the show-commands.
SHOW_DICT = {'show':{'callback': nullfunction, 'help_text': 'Help show', 'summary': 'Summary show', 'children':{
      'x':{'callback': nullfunction, 'help_text': 'Help show x', 'summary': 'Summary show x', 'children':{
      }},
      'y':{'callback': nullfunction, 'help_text': 'Help show y', 'summary': 'Summary show y', 'children':{
      }},
  }},
}


# This dictionary contains all the show-commands.
SHOW_X_DICT = {'show':{'children':{
      'x':{'callback': nullfunction, 'help_text': 'Help show x', 'summary': 'Summary show x', 'children':{
      }},
  }},
}

# This dictionary contains all the show-commands.
SHOW_DEFINED_Y_DICT = {'show':{'callback': nullfunction, 'help_text': 'Help show', 'summary': 'Summary show', 'children':{
      'y':{'callback': nullfunction, 'help_text': 'Help show y', 'summary': 'Summary show y', 'children':{
      }},
  }},
}



def branch_test():  
  testdict = deepcopy(COMBINED_DICT)
  seash_modules.remove_commanddict(testdict, SHOW_DICT)
  
  if not seash_modules.are_cmddicts_same(testdict, GET_DICT):
    raise Exception("Failed to remove entire command branch GET")
  
  # Repeat the same test the other way around
  testdict = deepcopy(COMBINED_DICT)
  seash_modules.remove_commanddict(testdict, GET_DICT)
  
  if not seash_modules.are_cmddicts_same(testdict, SHOW_DICT):
    raise Exception("Failed to remove entire command branch SHOW")



def singlecommand_test():
  # Removing a child node when there are other siblings must not have
  # affect the other siblings
  testdict = deepcopy(SHOW_DICT)
  seash_modules.remove_commanddict(testdict, SHOW_X_DICT)
  
  if not seash_modules.are_cmddicts_same(testdict, SHOW_DEFINED_Y_DICT):
    raise Exception("Failed to remove child node")
  
  # When removing a child node with a parent definition when there are other 
  # siblings, we should remove the parent definition as well. 
  testdict = deepcopy(SHOW_DICT)
  seash_modules.remove_commanddict(testdict, SHOW_DEFINED_Y_DICT)
  
  if not seash_modules.are_cmddicts_same(testdict, SHOW_X_DICT):
    raise Exception("Failed to remove parent node definition when removing child node")



singlecommand_test()
branch_test()


"""
Make sure that disabling a disabled module fails
"""

import seash

# We need to do module manipulation
import seash_dictionary
import seash_modules
#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

#pragma error Module is not enabled
commands = [
  "disable geoip"
]
seash.command_loop(commands)


"""
<Program Name>
  clearinghouse/command_callbacks.py

<Purpose>
  Defines all the command callbacks that are used for the Clearinghouse module.

  For more information on how the functions within this module are defined,
  refer to the main command_callbacks.py in the same directory as seash.py.

"""
import seash_helper
import seash_exceptions
import seash_global_variables

nmclient_createhandle = seash_helper.nmclient_createhandle
nmclient_getvesseldict = seash_helper.nmclient_getvesseldict
nmclient_destroyhandle = seash_helper.nmclient_destroyhandle

import seattleclearinghouse_xmlrpc

from repyportability import *
add_dy_support(locals())

dy_import_module_symbols('rsa.r2py')


is_printed_m2crypto_not_installed = False



# get # [type]
def get(input_dict, environment_dict):
  """
  <Purpose>
    Gets the specified vessels.

  <Arguments>
    input_dict:  The commanddict representing the user's input.
    environment_dict:  The dictionary representing the current seash
                       environment.

  <Side Effects>
    Connects to the Clearinghouse and acquires vessels.
    Adds the acquired vessels to the list of valid targets.

  <Exceptions>
    None

  <Returns>
    None
  """

  if environment_dict['currentkeyname'] is None or not seash_global_variables.keys[environment_dict['currentkeyname']]['privatekey']:
    raise seash_exceptions.UserError("Error, must get as an identity with a private key")

  # Activate secure mode if user did not specify the insecure keyword
  allow_ssl_insecure = _get_user_argument(input_dict, 'insecure') is not None
  vesselcount = int(_get_user_argument(input_dict, 'vesselcount'))

  try:
    vesseltype = _get_user_argument(input_dict, 'type')
  # User may not have specified a vesseltype
  except IndexError:
    vesseltype = None

  if not vesseltype in ['wan', 'lan', 'nat', None]:
    raise seash_exceptions.UserError("Error, unknown vessel type '"+vesseltype+"'")

  client = _connect_to_clearinghouse(environment_dict['currentkeyname'],
    allow_ssl_insecure)

  # Get the vessels!
  try:
    if vesseltype is None:
      vesseldicts = client.acquire_random_resources(vesselcount)
    else:
      vesseldicts = client.acquire_resources(vesseltype, vesselcount)

    _update_targets(vesseldicts, environment_dict)
  except (seattleclearinghouse_xmlrpc.UnableToAcquireResourcesError, seattleclearinghouse_xmlrpc.NotEnoughCreditsError), e:
    print str(e)


# release
def release(input_dict, environment_dict):
  """
  <Purpose>
    Releases the specified vessels.

  <Arguments>
    input_dict:  The commanddict representing the user's input.
    environment_dict:  The dictionary representing the current seash
                       environment.

  <Side Effects>
    Connects to the Clearinghouse and releases vessels.
    Removes the released vessels from the list of valid targets.
    Does not guarantee that all vessels specified are released!

  <Exceptions>
    None

  <Returns>
    None
  """
  # Activate secure mode if user did not specify the insecure keyword
  allow_ssl_insecure = _get_user_argument(input_dict, 'insecure') is not None

  # Get the group name to release
  groupname = environment_dict['currenttarget']
  nodelist = seash_global_variables.targets[groupname]

  # Get the Clearinghouse vessel handles for each vessel
  retdict = seash_helper.contact_targets(nodelist, _get_clearinghouse_vessel_handle)

  clearinghouse_vesselhandles = []
  faillist = []
  # parse the output so we can print out something intelligible
  for nodename in retdict:

    if retdict[nodename][0]:
      clearinghouse_vesselhandles.append(retdict[nodename][1])
    else:
      faillist.append(nodename)

  # Release!
  client = _connect_to_clearinghouse(environment_dict['currentkeyname'],
    allow_ssl_insecure)
  client.release_resources(clearinghouse_vesselhandles)

  # Remove each vessel from the targets list
  removed_nodehandles = seash_global_variables.targets[groupname][:]
  for handle in removed_nodehandles:
    for target in seash_global_variables.targets:
      if handle in seash_global_variables.targets[target]:
        seash_global_variables.targets[target].remove(handle)


# release group
def release_args(input_dict, environment_dict):
  """
  <Purpose>
    Releases the specified vessels.

  <Arguments>
    input_dict:  The commanddict representing the user's input.
    environment_dict:  The dictionary representing the current seash
                       environment.

  <Side Effects>
    Connects to the Clearinghouse and releases vessels.
    Removes the released vessels from the list of valid targets.
    Does not guarantee that all vessels specified are released!

  <Exceptions>
    None

  <Returns>
    None
  """
  # Activate secure mode if user did not specify the insecure keyword
  allow_ssl_insecure = _get_user_argument(input_dict, 'insecure') is not None

  # Get the group name to release
  groupname = _get_user_argument(input_dict, 'groupname')
  nodelist = seash_global_variables.targets[groupname]

  # Get the Clearinghouse vessel handles for each vessel
  retdict = seash_helper.contact_targets(nodelist, _get_clearinghouse_vessel_handle)

  clearinghouse_vesselhandles = []
  faillist = []
  # parse the output so we can print out something intelligible
  for nodename in retdict:

    if retdict[nodename][0]:
      clearinghouse_vesselhandles.append(retdict[nodename][1])
    else:
      faillist.append(nodename)

  # Release!
  client = _connect_to_clearinghouse(environment_dict['currentkeyname'],
    allow_ssl_insecure)
  client.release_resources(clearinghouse_vesselhandles)

  # Remove each vessel from the targets list
  removed_nodehandles = seash_global_variables.targets[groupname][:]
  for handle in removed_nodehandles:
    for target in seash_global_variables.targets:
      if handle in seash_global_variables.targets[target]:
        seash_global_variables.targets[target].remove(handle)


def _update_targets(vesseldicts, environment_dict):
  """
  <Purpose>
    Connects to the nodes in the vesseldicts and adds them to the list
    of valid targets.

  <Arguments>
    vesseldicts:
        A list of vesseldicts obtained through
        SeattleClearinghouseClient calls.

  <Side Effects>
    All valid targets that the user can access on the specified nodes
    are added to the list of targets.

  <Exceptions>
    None

  <Returns>
    None
  """
  # Compile a list of the nodes that we need to check
  nodelist = []
  for vesseldict in vesseldicts:
    nodeip_port = vesseldict['node_ip']+':'+str(vesseldict['node_port'])
    if not nodeip_port in nodelist:
      nodelist.append(nodeip_port)

  # we'll output a message about the new keys later...
  newidlist = []
  faillist = []

  # Clear the list so that the user doesn't target vessels acquired from
  # previous requests when targeting this group
  seash_global_variables.targets['acquired'] = []
  print nodelist

  # currently, if I browse more than once, I look up everything again...
  retdict = seash_helper.contact_targets(
    nodelist,
    seash_helper.browse_target,
    environment_dict['currentkeyname'],
    'acquired')

  # parse the output so we can print out something intelligible
  for nodename in retdict:
    if retdict[nodename][0]:
      newidlist = newidlist + retdict[nodename][1]
    else:
      faillist.append(nodename)

  seash_helper.print_vessel_errors(retdict)

  if len(newidlist) == 0:
    print "Could not add any new targets."
  else:
    print "Added targets: "+", ".join(newidlist)

  if len(seash_global_variables.targets['acquired']) > 0:
    num_targets = str(len(seash_global_variables.targets['acquired']))
    print "Added group 'acquired' with "+num_targets+" targets"


def _get_clearinghouse_vessel_handle(vesselhandle):
  """
  <Purpose>
    Acquires the unique vessel identifier for a given vesselhandle.

  <Arguments>
    vesselhandle:
      A vessel handle expressed in the form node_ip:node_port:vesselname.

  <Side Effects>
    Opens a connection to the vessel to retrieve its nodekey.

  <Exceptions>
    None

  <Returns>
    A list of Clearinghouse vesselhandles for each vessel.

  """
  host, portstring, vesselname = vesselhandle.split(':')
  port = int(portstring)

  # get information about the node's vessels
  try:
    nodehandle = nmclient_createhandle(host, port,
      timeout=seash_global_variables.globalseashtimeout)

  except NMClientException,e:
    return (False, str(e))

  try:
    # We need to get the nodekey on this vessel
    vesseldict = nmclient_getvesseldict(nodehandle)
  except NMClientException,e:
    return (False, str(e))
  finally:
    nmclient_destroyhandle(nodehandle)

  nodekeystr = rsa_publickey_to_string(vesseldict['nodekey'])
  return (True, nodekeystr+':'+vesselname)


def _connect_to_clearinghouse(identity, allow_ssl_insecure):
  try:
    return seattleclearinghouse_xmlrpc.SeattleClearinghouseClient(
      username=identity,
      private_key_string=rsa_privatekey_to_string(
        seash_global_variables.keys[identity]['privatekey']),
      allow_ssl_insecure=allow_ssl_insecure)

  except ImportError, e:
    if "M2Crypto" in str(e):
      raise seash_exceptions.UserError("You must have M2Crypto " +
        "installed to connect to the Clearinghouse securely.\n" +
        "For more information, run 'modulehelp clearinghouse'.")
    # Unknown error...
    raise

  except seattleclearinghouse_xmlrpc.SeattleClearinghouseError, e:
    if not allow_ssl_insecure and "No CA certs found in file" in str(e):
      print ("Your CA Certs file is corrupt or does not exist.\n" +
        "You need to download a CA certs file (you can do so here: \n" +
        "http://curl.haxx.se/ca/cacert.pem) and place it in the following location.")
      # Re-raise to get Seash to print out this error message
      raise seash_exceptions.UserError(str(e))
    # Unknown error...
    raise

def _get_user_argument(input_dict, argname):
  # Iterates through the dictionary to retrieve the user's argument
  command_key = input_dict.keys()[0]
  while input_dict[command_key]['name'] is not argname:
     input_dict = input_dict[command_key]['children']
     # No more children, argument couldn't be found.
     if not input_dict:
      return None
     command_key = input_dict.keys()[0]
  return command_key
